/*
Copyright (c) 2000-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description:
Define the multiple resource file reader class
for use in MultiRead1 and MultiRead2 examples 
*/



#ifndef __MultiRead_H
#define __MultiRead_H

class CMultipleResourceFileReader : public CBase
	{
public:
	// construct/destruct
	~CMultipleResourceFileReader();
	static CMultipleResourceFileReader* NewLC();
	// add resource file
	void AddResourceFileL(const TDesC& aName, TInt aVersion);
	HBufC8* AllocReadLC(TInt aResourceId);
private:
	// construct/destruct
	void ConstructL();
private:
	CArrayFixFlat<RResourceFile>* iResourceFiles;
	};

// construct/destruct

CMultipleResourceFileReader* CMultipleResourceFileReader::NewLC()
	{
	CMultipleResourceFileReader* self=new (ELeave) CMultipleResourceFileReader;
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

void CMultipleResourceFileReader::ConstructL()
	{
	iResourceFiles=new (ELeave) CArrayFixFlat<RResourceFile> (2);
		// new array of resource files
		// typically two in an application, so use granularity 2
	}

CMultipleResourceFileReader::~CMultipleResourceFileReader()
	{
	if (iResourceFiles)
		{
		for (TInt i=0; i < iResourceFiles->Count(); i++)
			(*iResourceFiles)[i].Close(); // close each resource file
		delete iResourceFiles; // and destroy container
		}
	}

// manipulate resource files and resources

void CMultipleResourceFileReader::AddResourceFileL(const TDesC& aName, TInt aVersion)
	{
	_LIT(KAddingResourceFile,"Adding resource file %S, version %d: will be file %d\n");
	console->Printf(KAddingResourceFile,
			&aName, aVersion, iResourceFiles->Count()
			);
	RResourceFile file;
	file.OpenL(fsSession,aName); // open resource file
	TRAPD(error,file.ConfirmSignatureL(aVersion)); // confirm its signature
	if (error!=KErrNone)
		{
		file.Close();
		User::Leave(error);
		}
	TRAP(error,iResourceFiles->AppendL(file))
	if (error!=KErrNone)
		{
		file.Close();
		User::Leave(error);
		}
	return;
	}

HBufC8* CMultipleResourceFileReader::AllocReadLC(TInt aResourceId)
	{
	_LIT(KReadingResource,"Reading resource %08X\n");
	_LIT(KOwnedByFile,"   Owned by file %d\n");

	console->Printf(KReadingResource,aResourceId);
	for (TInt i=0; i < iResourceFiles->Count(); i++)
		{ // scan all resource files to find owner
		RResourceFile& file=(*iResourceFiles)[i]; // get resource file
		if (!file.OwnsResourceId(aResourceId))
			continue; // continue if not owner
		console->Printf(KOwnedByFile,i);
		return file.AllocReadLC(aResourceId);
				// return resource from owning file (if present)
		}
	User::Leave(KErrNotFound);
	return 0; // can never be executed, but keeps compiler happy
	}

#endif
